/*
 * $Id: upsd.c,v 1.1 1995/11/08 19:24:28 alexis Exp $
 *
 * UPS Daemon
 * The Wild Wind Communications, 1995
 *
 * $Log: upsd.c,v $
 * Revision 1.1  1995/11/08  19:24:28  alexis
 * Initial revision
 *
 *
 */

#include <err.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/termios.h>
#include <sys/types.h>
#include <signal.h>

#include "config.h"
#include "smartups.h"

#define	U_ONLINE	0
#define U_BATTARIES	1

char *logfile		=	LOGFILE;
int shutdown_delay	=	SHUTDOWN_DELAY;

char *cmd_failed	=	CMD_FAILED;
char *cmd_restored	=	CMD_RESTORED;
char *cmd_shutdown	=	CMD_SHUTDOWN;
char *port_name		=	(char *)0;

int port;
int condition;

extern char readport(int);
extern void writeport(int, char *);
extern void usage(void);
extern void shutdown(int);
extern void execute(char *);

extern char *optarg;
extern int optind;

int main(argc, argv)
int argc;
char **argv;
{
	int c;
	struct termios t;

	while((c = getopt(argc, argv, "l:d:f:r:s:p:")) != EOF) {
		switch(c) {
			case 'l':
				logfile = optarg;
				break;
			case 'd':
				shutdown_delay = atoi(optarg);
				break;
			case 'f':
				cmd_failed = optarg;
				break;
			case 'r':
				cmd_restored = optarg;
				break;
			case 's':
				cmd_shutdown = optarg;
				break;
			case 'p':
				port_name = optarg;
				break;
			case '?':
			default:
			     usage();
		}
	}

	argc -= optind;
	argv += optind;

	if((int)signal(SIGALRM, shutdown) == -1) {
		errx(1, "can't set signal handler: %s", strerror(errno));
	}

	if(!port_name) {
		errx(1, "no port name specified");
	}
	if((port = open(port_name, O_RDWR)) == -1) {
		errx(1, "%s: %s", port_name, strerror(errno));
	}


	if((tcgetattr(port, &t)) == -1) {
		errx(1, "%s", strerror(errno));
	};

	t.c_iflag	= 0;
	t.c_oflag	= 0;
	t.c_cflag	= CS8 | CREAD | CLOCAL;
	t.c_lflag	= 0;
	t.c_ispeed	= B2400;
	t.c_ospeed	= B2400;
	memset(&t.c_cc, '\0', sizeof(t.c_cc));

	if((tcsetattr(port, TCSANOW, &t)) == -1) {
		errx(1, "%s", strerror(errno));
	};

	condition = U_ONLINE;

	daemon();

	if(errno) {
		errx(1, "%s", strerror(errno));
	}

	writeport(port, "Y");
	for(;;) {
		c = readport(port);

		switch(c) {
			case '!':
				if(condition != U_BATTARIES) {
					condition = U_BATTARIES;
					alarm(shutdown_delay);
					execute(cmd_failed);
				}
				break;

			case '$':
				if(condition != U_ONLINE) {
					condition = U_ONLINE;
					execute(cmd_restored);
				}
				alarm(0); /* safeguard */
				break;

			default:
				break;
		};
	}
}

char readport(port)
int port;
{
	char c;
	if(read(port, &c, 1) == -1) {
		errx(1, "can't read from %s: %s", port_name, strerror(errno));
	};
	return c;
}

void writeport(port, s)
int port;
char *s;
{
	if(write(port, s, strlen(s)) == -1) {
		errx(1, "can't write to %s: %s", port_name, strerror(errno));
	};
}

void usage(void) {
	fprintf(stderr, "usage: upsd [-l logfile] [-d delay] [-f fail_cmd]\n\t"
			"[-r restore_cmd] [-s shutdown_cmd] [-p port]\n\n");
	exit(1);
}

void shutdown(sig)
int sig;
{
	register int i, j;

	writeport(port, U_SHUTDOWN);
	writeport(port, U_SHUTDOWN);

	execute(cmd_shutdown);

	for(;;) {
		sleep(SHUTDOWN_SEND_DELAY);
		writeport(port, U_SHUTDOWN);
	};
};

void execute(command)
char *command;
{
	pid_t pid;

	pid = fork();
	switch(pid) {
		case 0:	/* child */
			system(command);
			exit(0);
		case -1:
			errx(1, "unable to execute command: %s",
				strerror(errno));
			break;
		default:
			break;
	}
}
